#!/usr/bin/gawk -f
#
# Usage: generate_table.awk < ${board}.txt
#
# Takes the file generated by collect.sh and generates an ASCII
# table that can be inserted into the README.md.

BEGIN {
  NUM_ACE_CRC_ALGORITHMS = 9
  NUM_ALGORITHMS = 12
  labels[0] = "baseline"
  labels[1] = "crc8_bit";
  labels[2] = "crc8_nibble";
  labels[3] = "crc8_byte";
  labels[4] = "crc16ccitt_bit";
  labels[5] = "crc16ccitt_nibble";
  labels[6] = "crc16ccitt_byte";
  labels[7] = "crc32_bit";
  labels[8] = "crc32_nibble";
  labels[9] = "crc32_byte";
  labels[10] = "CRC32";
  labels[11] = "Arduino_CRC32";
  labels[12] = "FastCRC";
  record_index = 0
}
{
  u[record_index]["flash"] = $2
  u[record_index]["ram"] = $4
  record_index++
}
END {
  # Calculate the flash and memory deltas from baseline
  base_flash = u[0]["flash"]
  base_ram = u[0]["ram"]
  for (i = 0; i <= NUM_ALGORITHMS; i++) {
    u[i]["d_flash"] = u[i]["flash"]- base_flash
    u[i]["d_ram"] = u[i]["ram"]- base_ram
  }

  printf("+--------------------------------------------------------------+\n")
  printf("| functionality                   |  flash/  ram |       delta |\n")
  printf("|---------------------------------+--------------+-------------|\n")
  printf("| %-31s | %6d/%5d | %5d/%5d |\n",
    labels[0], u[0]["flash"], u[0]["ram"], u[0]["d_flash"], u[0]["d_ram"])
  printf("|---------------------------------+--------------+-------------|\n")
  for (i = 1; i <= NUM_ACE_CRC_ALGORITHMS; i++) {
    printf("| %-31s | %6d/%5d | %5d/%5d |\n",
        labels[i], u[i]["flash"], u[i]["ram"], u[i]["d_flash"], u[i]["d_ram"])
  }
  printf("|---------------------------------+--------------+-------------|\n")
  for (i = NUM_ACE_CRC_ALGORITHMS + 1; i <= NUM_ALGORITHMS; i++) {
    printf("| %-31s | %6d/%5d | %5d/%5d |\n",
        labels[i], u[i]["flash"], u[i]["ram"], u[i]["d_flash"], u[i]["d_ram"])
  }
  printf("+--------------------------------------------------------------+\n")
}
